import React, { useRef } from 'react';
import {
  ActionType,
  PageContainer,
  ProForm,
  ProFormDatePicker,
  ProFormDateTimePicker,
  ProFormDependency,
  ProFormSelect,
  ProFormText,
  ProFormTextArea,
  ProTable,
} from '@ant-design/pro-components';
import { Button, Card, Modal } from 'antd';
import { AddOutline } from 'antd-mobile-icons';
import { EditBundle, useEdit } from '@/core/hooks/useEdit';
import dayjs from 'dayjs';
import ReactJson from 'react-json-view';
import { getConfigGetApi, postConfigPageApi, postConfigSaveApi } from '@/services/swagger/configAdmin';

enum ConfigPermission {
  /**
   * 普通配置(游客/授权访问)
   */
  NORMAL = 10,
  /**
   * 业务配置(授权访问)
   */
  BUSINESS = 30,
  /**
   * 系统配置(不可访问,不可修改)
   */
  SYSTEM = 60,
}

type ConfigType = 'int' | 'long' | 'str' | 'bool' | 'date' | 'dateTime' | 'json' | 'array';


const Edit: React.FC<{ bundle: EditBundle<string>; onSuccess?: () => void }> = (props) => {
  const { bundle } = props;
  const [form] = ProForm.useForm();

  return (
    <Modal
      open={bundle.open}
      title={bundle.id ? '编辑配置' : '添加配置'}
      onOk={async () => {
        const values = await form.validateFields();

        console.log(values);

        const res = await postConfigSaveApi({
          ...values,
        });

        if (!res.success) {
          return;
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onCancel={() => {
        bundle.close();
      }}
    >
      <ProForm<Partial<SectApi.ConfigVO>>
        className={'mt-5'}
        validateTrigger={false}
        form={form}
        submitter={false}
        layout={'horizontal'}
        labelCol={{ span: 4 }}
        request={async (): Promise<Partial<SectApi.ConfigVO>> => {
          if (bundle.id) {
            const res = await getConfigGetApi({code: bundle.id})

            return {
              code: res.data.code,
              type: res.data.type as ConfigType,
              remark: res.data.remark,
              content: res.data.content,
            };
          } else {
            return {
              type: 'str',
            };
          }
        }}
      >
        <ProFormText
          disabled={bundle.id !== undefined}
          name={'code'}
          label={'编码'}
          rules={[
            {
              required: true,
              message: '请输入配置编码',
            },
          ]}
        />
        <ProFormSelect
          fieldProps={{ allowClear: false }}
          rules={[
            {
              required: true,
              message: '请选择权限',
            },
          ]}
          name={'permission'}
          label={'权限'}
          options={[
            {
              label: '普通配置(游客/授权访问)',
              value: ConfigPermission.NORMAL,
            },
            {
              label: '业务配置(授权访问)',
              value: ConfigPermission.BUSINESS,
            },
          ]}
          initialValue={ConfigPermission.NORMAL}
        />
        <ProFormText name={'remark'} label={'说明'} />
        <ProFormSelect
          fieldProps={{
            allowClear: false,
          }}
          name={'type'}
          label={'类型'}
          rules={[
            {
              required: true,
              message: '请选择配置类型',
            },
          ]}
          options={[
            {
              label: '整数',
              value: 'int',
            },
            {
              label: '长整数',
              value: 'long',
            },
            {
              label: '字符串',
              value: 'str',
            },
            {
              label: '布尔',
              value: 'bool',
            },
            {
              label: '日期',
              value: 'date',
            },
            {
              label: '日期时间',
              value: 'dateTime',
            },
            {
              label: 'JSON',
              value: 'json',
            },
            {
              label: '数组',
              value: 'array',
            },
          ]}
        ></ProFormSelect>
        <ProFormDependency name={['type']}>
          {({ type }) => {
            const which = type as ConfigType;

            switch (which) {
              case 'long':
              case 'int':
                return (
                  <ProFormText
                    name={'content'}
                    label={'内容'}
                    placeholder={'请输入整数'}
                    fieldProps={{
                      type: 'number',
                    }}
                    initialValue={0}
                    rules={[
                      {
                        required: true,
                        message: '请输入整数',
                      },
                    ]}
                  />
                );
              case 'str':
                return (
                  <ProFormText
                    name={'content'}
                    label={'内容'}
                    placeholder={'请输入字符串'}
                    initialValue={''}
                    rules={[
                      {
                        required: true,
                        message: '请输入字符串',
                      },
                      {
                        min: 1,
                        message: '字符串长度不能小于1',
                      },
                    ]}
                  />
                );
              case 'bool':
                return (
                  <ProFormSelect
                    fieldProps={{
                      allowClear: false,
                    }}
                    rules={[
                      {
                        required: true,
                        message: '请选择布尔值',
                      },
                    ]}
                    initialValue={true}
                    name={'content'}
                    label={'内容'}
                    options={[
                      {
                        label: '是',
                        value: 'true',
                      },
                      {
                        label: '否',
                        value: 'false',
                      },
                    ]}
                  ></ProFormSelect>
                );
              case 'date':
                form.setFieldValue('content', new Date());
                return (
                  <ProFormDatePicker
                    name={'content'}
                    label={'内容'}
                    rules={[
                      {
                        required: true,
                        message: '请选择日期',
                      },
                    ]}
                  />
                );
              case 'dateTime':
                form.setFieldValue('content', new Date());
                return (
                  <ProFormDateTimePicker
                    name={'content'}
                    label={'内容'}
                    rules={[
                      {
                        required: true,
                        message: '请选择日期时间',
                      },
                    ]}
                  />
                );
              case 'json':
                form.setFieldValue('content', '{}');
                return (
                  <ProFormTextArea
                    name={'content'}
                    label={'内容'}
                    placeholder={'请输入JSON'}
                    rules={[
                      {
                        required: true,
                        message: '请输入JSON',
                      },
                      () => ({
                        validator(_, value) {
                          try {
                            const v = JSON.parse(value);

                            if (Array.isArray(v)) {
                              return Promise.reject('JSON不能是数组');
                            }

                            if (typeof v !== 'object') {
                              return Promise.reject('JSON必须是对象');
                            }

                            return Promise.resolve();
                          } catch (e) {
                            return Promise.reject('请输入正确的JSON');
                          }
                        },
                      }),
                    ]}
                  />
                );
              case 'array':
                form.setFieldValue('content', '[]');
                return (
                  <ProFormTextArea
                    name={'content'}
                    label={'内容'}
                    placeholder={'请输入数组'}
                    rules={[
                      {
                        required: true,
                        message: '请输入数组',
                      },
                      () => ({
                        validator(_, value) {
                          try {
                            const v = JSON.parse(value);

                            if (!Array.isArray(v)) {
                              return Promise.reject('内容必须是数组');
                            }

                            return Promise.resolve();
                          } catch (e) {
                            return Promise.reject('请输入正确的数组');
                          }
                        },
                      }),
                    ]}
                  />
                );
              default:
                return <></>;
            }
          }}
        </ProFormDependency>
      </ProForm>
    </Modal>
  );
};

const JSONView: React.FC<{ value: string }> = (props) => {
  const [open, setOpen] = React.useState(false);

  return (
    <>
      {open && (
        <Modal
          title={'JSON'}
          open={open}
          footer={false}
          onCancel={() => setOpen(false)}
        >
          <ReactJson src={JSON.parse(props.value)}></ReactJson>
        </Modal>
      )}
      <a
        onClick={() => {
          setOpen(true);
        }}
      >
        查看
      </a>
    </>
  );
};

const Settings: React.FC = () => {
  const edit = useEdit<string>();
  const ref = useRef<ActionType>();

  return (
    <PageContainer>
      {edit.open && (
        <Edit
          bundle={edit}
          onSuccess={() => {
            ref.current?.reload();
          }}
        />
      )}
      <Card>
        <ProTable<SectApi.ConfigVO>
          actionRef={ref}
          search={false}
          columns={[
            {
              title: '编码',
              dataIndex: 'code',
            },
            {
              title: '类型',
              dataIndex: 'type',
              render: (_, record) => {
                switch (record.type) {
                  case 'int':
                    return '整数';
                  case 'long':
                    return '长整数';
                  case 'str':
                    return '字符串';
                  case 'bool':
                    return '布尔';
                  case 'date':
                    return '日期';
                  case 'dateTime':
                    return '日期时间';
                  case 'json':
                    return 'JSON';
                  case 'array':
                    return '数组';
                  default:
                    return '未知';
                }
              },
            },

            {
              title: '说明',
              dataIndex: 'remark',
            },
            {
              title: '内容',
              dataIndex: 'content',
              render: (_, record) => {
                switch (record.type) {
                  case 'json':
                  case 'array':
                    return record.content &&  <JSONView value={record.content}></JSONView>;
                  case 'date':
                    return dayjs(record.content).format('YYYY-MM-DD');
                  case 'dateTime':
                    return dayjs(record.content).format('YYYY-MM-DD HH:mm:ss');
                  default:
                    return record.content;
                }
              },
            },
            {
              title: '权限',
              dataIndex: 'permission',
              render: (_, record) => {
                switch (record.permission) {
                  case ConfigPermission.NORMAL:
                    return '普通配置';
                  case ConfigPermission.BUSINESS:
                    return '业务配置';
                  case ConfigPermission.SYSTEM:
                    return '系统配置';
                  default:
                    return '未知';
                }
              },
            },
            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => {
                if(!record.code) {
                  return null
                }

                const opts = [];

                if (record.permission !== ConfigPermission.SYSTEM) {
                  opts.push(
                    <a
                      key={'edit'}
                      onClick={() => {
                        if(record.code)
                        {
                          edit.update(record.code);
                        }
                      }}
                    >
                      编辑
                    </a>,
                  );
                }

                return opts;
              },
            },
          ]}
          toolbar={{
            actions: [
              <Button
                type={'primary'}
                onClick={() => {
                  edit.create();
                }}
                icon={<AddOutline />}
                key={'add'}
              >
                添加
              </Button>,
            ],
            settings: [],
          }}
          request={async (params) => {
            const res = await postConfigPageApi({
              current:params.current || 1,
              size:params.pageSize || 20
            });

            return {
              data: res.data.records,
              total: res.data.current,
              success: res.success,
            };
          }}
        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default Settings;
