import {
  ActionType,
  PageContainer,
  ProForm,
  ProFormText, ProFormTreeSelect,
  ProTable,
  RequestOptionsType,
} from '@ant-design/pro-components';
import {Button, Card, Modal} from 'antd';
import React, {useRef} from 'react';
import {AddOutline} from 'antd-mobile-icons';
import {EditBundle, useEdit} from '@/core/hooks/useEdit';
import {deleteRoleRemoveApi, getRoleDetailsApi, postRolePageApi, postRoleSaveApi} from '@/services/swagger/roleAdmin';
import {postMenuTreeListApi} from '@/services/swagger/menuAdmin';

const Edit: React.FC<{ bundle: EditBundle; onSuccess?: () => void }> = (props) => {
  const {bundle} = props;
  const [form] = ProForm.useForm<SectApi.RoleSaveQuery>();
  const [loading, setLoading] = React.useState(false);

  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑角色' : '添加角色'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);
          await postRoleSaveApi({
            id: bundle.id,
            ...form.getFieldsValue(),
          })

        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{span: 4}}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');

            const res = await getRoleDetailsApi({id: bundle.id})


            const menuIds: number[] = []

            function getMenuIds(menus?: SectApi.MenuVO[]) {
              menus?.forEach((menu) => {
                if (menu.id) {
                  menuIds.push(menu.id)
                }

                if (menu.children) {
                  getMenuIds(menu.children)

                }
              })
            }

            getMenuIds(res.data.menus)

            return {
              ...res.data,
              menuIds: menuIds
            };
          }

          return Promise.resolve({});
        }}
      >
        <ProFormText
          name="name"
          label={'角色名称'}
          rules={[
            {
              required: true,
              message: '请输入角色名称',
            },
            {
              min: 3,
              max: 30,
              message: '角色名称长度必须在3-30之间',
            },
          ]}
          placeholder={'请输入角色名称'}
        ></ProFormText>

        <ProFormTreeSelect
          fieldProps={{
            'multiple': true,
          }}
          label='父菜单'
          name='menuIds'
          style={{width: '100%'}}
          placeholder="请选择菜单"
          allowClear
          request={async () => {
            const transformToTreeData = (data: SectApi.MenuVO[]): RequestOptionsType[] =>
              data.map<RequestOptionsType>((item) => ({
                label: item.name,
                value: item.id,
                children: item.children ? transformToTreeData(item.children) : [],
              }));
            const rs = await postMenuTreeListApi({});
            const treeData = transformToTreeData(rs.data);
            console.log(treeData)
            return treeData
          }}
        >
        </ProFormTreeSelect>

      </ProForm>
    </Modal>
  );
};

const RoleExp: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();
  return (
    <PageContainer content={''}>
      {bundle.open && <Edit bundle={bundle} onSuccess={() => ref.current?.reload()}></Edit>}
      <Card>
        <ProTable<SectApi.RoleVO>
          rowKey={'id'}
          request={async (params) => {
            const res = await postRolePageApi({
              current: params.current || 1,
              size: params.pageSize || 10,
            })

            return {
              success: res.success,
              total: res.data.total,
              data: res.data.records,
            };
          }}
          columns={[
            {
              title: 'ID',
              dataIndex: 'id',
            },
            {
              title: '角色名称',
              dataIndex: 'name',
            },
            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => [
                <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                  编辑
                </a>,
                <a
                  key={'delete'}
                  onClick={async () => {
                    if (record.id) {
                      await deleteRoleRemoveApi({id: record.id})
                      ref.current?.reload();
                    }
                  }}
                >
                  删除
                </a>,
              ],
            },
          ]}
          actionRef={ref}
          toolbar={{
            actions: [
              <Button icon={<AddOutline/>} key={'add'} type={'primary'} onClick={bundle.create}>
                添加
              </Button>,
            ],
            settings: [],
          }}
          search={false}
        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default RoleExp;
