import {ActionType, PageContainer, ProForm, ProFormSelect, ProFormText, ProTable,} from '@ant-design/pro-components';
import {Button, Card, Modal} from 'antd';
import React, {useRef} from 'react';
import {AddOutline} from 'antd-mobile-icons';
import {EditBundle, useEdit} from '@/core/hooks/useEdit';
import { deleteStaffRemoveApi, getStaffDetailsApi, postStaffPageApi, postStaffSaveApi } from '@/services/swagger/staffAdmin';
import { postRolePageApi } from '@/services/swagger/roleAdmin';

const Edit: React.FC<{ bundle: EditBundle; onSuccess?: () => void }> = (props) => {
  const {bundle} = props;
  const [form] = ProForm.useForm<SectApi.StaffSaveQuery>();
  const [loading, setLoading] = React.useState(false);

  return (
    <Modal
      confirmLoading={loading}
      title={bundle.id ? '编辑管理员' : '添加管理员'}
      open={bundle.open}
      onOk={async () => {
        await form.validateFields();

        try {
          setLoading(true);

          await postStaffSaveApi({
            id: bundle.id,
            ...form.getFieldsValue(),
            thumbAvatar: '',
            level: 1,
          });
        } finally {
          setLoading(false);
        }

        props.onSuccess?.();
        bundle.close();
      }}
      onCancel={() => {
        form.resetFields();
        bundle.close();
      }}
    >
      <ProForm
        className={'mt-5'}
        form={form}
        labelCol={{span: 4}}
        layout={'horizontal'}
        submitter={false}
        request={async () => {
          if (bundle.id !== undefined) {
            console.log(bundle.id, 'bundle');
            const res = await getStaffDetailsApi({id: bundle.id});

            return {
              ...res.data,
              roleIds: res.data.roles?.map((rule) => {
                return rule.id
              })
            }
          }

          return Promise.resolve({});
        }}
      >
        <ProFormText
          name="username"
          label={'账号'}
          rules={[
            {
              required: true,
              message: '请输入账号',
            },
            {
              min: 3,
              max: 20,
              message: '账号长度必须在3-20之间',
            },
          ]}
          placeholder={'请输入账号'}
        ></ProFormText>
        <ProFormText
          name="nickname"
          label="姓名"
          rules={[
            {
              required: true,
              message: '请输入姓名',
            },
            {
              min: 2,
              max: 20,
              message: '姓名长度必须在2-20之间',
            },
          ]}
          placeholder={'请输入用户姓名'}
        />
        <ProFormText.Password
          name="password"
          label="密码"
          placeholder={'请输入密码'}
          rules={[
            {
              required: true,
              message: '请输入密码',
            },
            {
              min: 6,
              max: 50,
              message: '密码长度必须在6-50之间',
            },
          ]}
        />
        <ProFormSelect
          fieldProps={{
            'mode': 'multiple',
          }}
          name="roleIds"
          label="角色"
          request={async () => {
            const res = await postRolePageApi({        current: 1,
              size: 1000});
              const records = res.data.records ?? []
              return records.map(record => ({
                label: record.name,
                value: record.id,
              }))
          }}
          placeholder="请选择角色"
          rules={[{required: true, message: '请选择角色'}]}
        />
      </ProForm>
    </Modal>
  );
};

const Staff: React.FC = () => {
  const bundle = useEdit();
  const ref = useRef<ActionType>();



  return (
    <PageContainer content={''}>
      {bundle.open && <Edit bundle={bundle} onSuccess={() => ref.current?.reload()}></Edit>}
      <Card>
        <ProTable<SectApi.StaffVO>
          rowKey={'id'}

          request={async (params, ...rest) => {
            console.log(params, ...rest)

            const res = await postStaffPageApi(

              {
                current: params.current || 1,
                size: params.pageSize || 10,
              }
              );

            return {
              success: res.success,
              total: res.data.total,
              data: res.data.records,
            };
          }}
          columns={[
            {
              title: 'ID',
              dataIndex: 'id',
            },
            {
              title: '账号',
              dataIndex: 'username',
              filters: true,
            },
            {
              title: '姓名',
              dataIndex: 'nickname',
              copyable: true,
              ellipsis: true,
            },
            {
              title: '角色列表',
              dataIndex: 'roles',
              // @ts-ignore
              render: (roles: SectApi.RoleVO[]) => {
                return roles.map(role => role.name).join(', ')
              },
            },
            {
              title: '操作',
              valueType: 'option',
              render: (_, record) => [
                <a key={'edit'} onClick={() => bundle.update(record.id as number)}>
                  编辑
                </a>,
                <a
                  key={'delete'}
                  onClick={async () => {
                    if(record.id) {
                      await deleteStaffRemoveApi({id: record.id})
                      ref.current?.reload();
                    }
                  }}
                >
                  删除
                </a>,
              ],
            },
          ]
          }
          search={{
            labelWidth: 'auto',
          }}
          form={{
            ignoreRules: false,
          }}


          actionRef={ref}
          toolbar={{
            actions: [
              <Button icon={<AddOutline/>} key={'add'} type={'primary'} onClick={bundle.create}>
                添加
              </Button>,
            ],
            settings: [],
          }}

        ></ProTable>
      </Card>
    </PageContainer>
  );
};

export default Staff;
